@@ -2,7 +2,7 @@ from dataclasses import asdict
 from pathlib import Path
 from typing import TYPE_CHECKING, Optional, Union
 
-from agno.db.base import BaseDb
+from agno.db.base import AsyncBaseDb, BaseDb
 from agno.db.schemas.evals import EvalRunRecord, EvalType
 from agno.utils.log import log_debug, logger
 
@@ -49,7 +49,7 @@ def log_eval_run(
 
 
 async def async_log_eval(
-    db: BaseDb,
+    db: Union[BaseDb, AsyncBaseDb],
     run_id: str,
     run_data: dict,
     eval_type: EvalType,
@@ -65,21 +65,38 @@ async def async_log_eval(
     """Call the API to create an evaluation run."""
 
     try:
-        db.create_eval_run(
-            EvalRunRecord(
-                run_id=run_id,
-                eval_type=eval_type,
-                eval_data=run_data,
-                eval_input=eval_input,
-                agent_id=agent_id,
-                model_id=model_id,
-                model_provider=model_provider,
-                name=name,
-                evaluated_component_name=evaluated_component_name,
-                team_id=team_id,
-                workflow_id=workflow_id,
+        if isinstance(db, AsyncBaseDb):
+            await db.create_eval_run(
+                EvalRunRecord(
+                    run_id=run_id,
+                    eval_type=eval_type,
+                    eval_data=run_data,
+                    eval_input=eval_input,
+                    agent_id=agent_id,
+                    model_id=model_id,
+                    model_provider=model_provider,
+                    name=name,
+                    evaluated_component_name=evaluated_component_name,
+                    team_id=team_id,
+                    workflow_id=workflow_id,
+                )
+            )
+        else:
+            db.create_eval_run(
+                EvalRunRecord(
+                    run_id=run_id,
+                    eval_type=eval_type,
+                    eval_data=run_data,
+                    eval_input=eval_input,
+                    agent_id=agent_id,
+                    model_id=model_id,
+                    model_provider=model_provider,
+                    name=name,
+                    evaluated_component_name=evaluated_component_name,
+                    team_id=team_id,
+                    workflow_id=workflow_id,
+                )
             )
-        )
     except Exception as e:
         log_debug(f"Could not create agent event: {e}")
 
