@@ -1,4 +1,4 @@
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from os import getenv
 from typing import Any, Dict, Optional
 
@@ -24,7 +24,7 @@ class AIMLAPI(OpenAILike):
     name: str = "AIMLAPI"
     provider: str = "AIMLAPI"
 
-    api_key: Optional[str] = getenv("AIMLAPI_API_KEY")
+    api_key: Optional[str] = field(default_factory=lambda: getenv("AIMLAPI_API_KEY"))
     base_url: str = "https://api.aimlapi.com/v1"
     max_tokens: int = 4096
 
