@@ -1 +1,5 @@
 from agno.models.deepseek.deepseek import DeepSeek
+
+__all__ = [
+    "DeepSeek",
+]
