@@ -1 +1,5 @@
 from agno.models.perplexity.perplexity import Perplexity
+
+__all__ = [
+    "Perplexity",
+]
