@@ -1,4 +1,4 @@
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from os import getenv
 from typing import Optional
 
@@ -22,7 +22,7 @@ class Sambanova(OpenAILike):
     name: str = "Sambanova"
     provider: str = "Sambanova"
 
-    api_key: Optional[str] = getenv("SAMBANOVA_API_KEY")
+    api_key: Optional[str] = field(default_factory=lambda: getenv("SAMBANOVA_API_KEY"))
     base_url: str = "https://api.sambanova.ai/v1"
 
     supports_native_structured_outputs: bool = False
