@@ -1,18 +1,22 @@
 from abc import ABC, abstractmethod
-from typing import Optional
+from typing import List, Optional
 
 from fastapi import APIRouter
 
 from agno.agent import Agent
 from agno.team import Team
+from agno.workflow.workflow import Workflow
 
 
 class BaseInterface(ABC):
     type: str
     version: str = "1.0"
-    router_prefix: str = ""
     agent: Optional[Agent] = None
     team: Optional[Team] = None
+    workflow: Optional[Workflow] = None
+
+    prefix: str
+    tags: List[str]
 
     router: APIRouter
 
