@@ -3,41 +3,24 @@ import math
 from typing import Callable, List
 
 from agno.tools import Toolkit
-from agno.utils.log import log_info, logger
+from agno.utils.log import log_debug, logger
 
 
 class CalculatorTools(Toolkit):
     def __init__(
         self,
-        add: bool = True,
-        subtract: bool = True,
-        multiply: bool = True,
-        divide: bool = True,
-        exponentiate: bool = False,
-        factorial: bool = False,
-        is_prime: bool = False,
-        square_root: bool = False,
-        enable_all: bool = False,
         **kwargs,
     ):
-        # Build the include_tools list based on enabled functions
-        tools: List[Callable] = []
-        if add or enable_all:
-            tools.append(self.add)
-        if subtract or enable_all:
-            tools.append(self.subtract)
-        if multiply or enable_all:
-            tools.append(self.multiply)
-        if divide or enable_all:
-            tools.append(self.divide)
-        if exponentiate or enable_all:
-            tools.append(self.exponentiate)
-        if factorial or enable_all:
-            tools.append(self.factorial)
-        if is_prime or enable_all:
-            tools.append(self.is_prime)
-        if square_root or enable_all:
-            tools.append(self.square_root)
+        tools: List[Callable] = [
+            self.add,
+            self.subtract,
+            self.multiply,
+            self.divide,
+            self.exponentiate,
+            self.factorial,
+            self.is_prime,
+            self.square_root,
+        ]
 
         # Initialize the toolkit with auto-registration enabled
         super().__init__(name="calculator", tools=tools, **kwargs)
@@ -53,7 +36,7 @@ class CalculatorTools(Toolkit):
             str: JSON string of the result.
         """
         result = a + b
-        log_info(f"Adding {a} and {b} to get {result}")
+        log_debug(f"Adding {a} and {b} to get {result}")
         return json.dumps({"operation": "addition", "result": result})
 
     def subtract(self, a: float, b: float) -> str:
@@ -67,7 +50,7 @@ class CalculatorTools(Toolkit):
             str: JSON string of the result.
         """
         result = a - b
-        log_info(f"Subtracting {b} from {a} to get {result}")
+        log_debug(f"Subtracting {b} from {a} to get {result}")
         return json.dumps({"operation": "subtraction", "result": result})
 
     def multiply(self, a: float, b: float) -> str:
@@ -81,7 +64,7 @@ class CalculatorTools(Toolkit):
             str: JSON string of the result.
         """
         result = a * b
-        log_info(f"Multiplying {a} and {b} to get {result}")
+        log_debug(f"Multiplying {a} and {b} to get {result}")
         return json.dumps({"operation": "multiplication", "result": result})
 
     def divide(self, a: float, b: float) -> str:
@@ -101,7 +84,7 @@ class CalculatorTools(Toolkit):
             result = a / b
         except Exception as e:
             return json.dumps({"operation": "division", "error": str(e), "result": "Error"})
-        log_info(f"Dividing {a} by {b} to get {result}")
+        log_debug(f"Dividing {a} by {b} to get {result}")
         return json.dumps({"operation": "division", "result": result})
 
     def exponentiate(self, a: float, b: float) -> str:
@@ -115,7 +98,7 @@ class CalculatorTools(Toolkit):
             str: JSON string of the result.
         """
         result = math.pow(a, b)
-        log_info(f"Raising {a} to the power of {b} to get {result}")
+        log_debug(f"Raising {a} to the power of {b} to get {result}")
         return json.dumps({"operation": "exponentiation", "result": result})
 
     def factorial(self, n: int) -> str:
@@ -131,7 +114,7 @@ class CalculatorTools(Toolkit):
             logger.error("Attempt to calculate factorial of a negative number")
             return json.dumps({"operation": "factorial", "error": "Factorial of a negative number is undefined"})
         result = math.factorial(n)
-        log_info(f"Calculating factorial of {n} to get {result}")
+        log_debug(f"Calculating factorial of {n} to get {result}")
         return json.dumps({"operation": "factorial", "result": result})
 
     def is_prime(self, n: int) -> str:
@@ -164,5 +147,5 @@ class CalculatorTools(Toolkit):
             return json.dumps({"operation": "square_root", "error": "Square root of a negative number is undefined"})
 
         result = math.sqrt(n)
-        log_info(f"Calculating square root of {n} to get {result}")
+        log_debug(f"Calculating square root of {n} to get {result}")
         return json.dumps({"operation": "square_root", "result": result})
