@@ -124,20 +124,6 @@ class GmailTools(Toolkit):
         self.scopes = scopes or self.DEFAULT_SCOPES
         self.port = port
 
-        """ tools functions:
-         enable_get_latest_emails (bool): Enable getting latest emails.
-         enable_get_emails_from_user (bool): Enable getting emails from specific user.
-         enable_get_unread_emails (bool): Enable getting unread emails.
-         enable_get_starred_emails (bool): Enable getting starred emails.
-         enable_get_emails_by_context (bool): Enable getting emails by context.
-         enable_get_emails_by_date (bool): Enable getting emails by date.
-         enable_get_emails_by_thread (bool): Enable getting emails by thread.
-         enable_create_draft_email (bool): Enable creating draft emails.
-         enable_send_email (bool): Enable sending emails.
-         enable_send_email_reply (bool): Enable sending email replies.
-         all (bool): Enable all tools.
-        """
-
         tools: List[Any] = [
             # Reading emails
             self.get_latest_emails,
@@ -148,6 +134,9 @@ class GmailTools(Toolkit):
             self.get_emails_by_date,
             self.get_emails_by_thread,
             self.search_emails,
+            # Email management
+            self.mark_email_as_read,
+            self.mark_email_as_unread,
             # Composing emails
             self.create_draft_email,
             self.send_email,
@@ -173,12 +162,18 @@ class GmailTools(Toolkit):
             "get_emails_by_thread",
             "search_emails",
         ]
+        modify_operations = ["mark_email_as_read", "mark_email_as_unread"]
         if any(read_operation in self.functions for read_operation in read_operations):
             read_scope = "https://www.googleapis.com/auth/gmail.readonly"
             write_scope = "https://www.googleapis.com/auth/gmail.modify"
             if read_scope not in self.scopes and write_scope not in self.scopes:
                 raise ValueError(f"The scope {read_scope} is required for email reading operations")
 
+        if any(modify_operation in self.functions for modify_operation in modify_operations):
+            modify_scope = "https://www.googleapis.com/auth/gmail.modify"
+            if modify_scope not in self.scopes:
+                raise ValueError(f"The scope {modify_scope} is required for email modification operations")
+
     def _auth(self) -> None:
         """Authenticate with Gmail API"""
         token_file = Path(self.token_path or "token.json")
@@ -555,6 +550,56 @@ class GmailTools(Toolkit):
         except Exception as error:
             return f"Unexpected error retrieving emails with query '{query}': {type(error).__name__}: {error}"
 
+    @authenticate
+    def mark_email_as_read(self, message_id: str) -> str:
+        """
+        Mark a specific email as read by removing the 'UNREAD' label.
+        This is crucial for long polling scenarios to prevent processing the same email multiple times.
+
+        Args:
+            message_id (str): The ID of the message to mark as read
+
+        Returns:
+            str: Success message or error description
+        """
+        try:
+            # Remove the UNREAD label to mark the email as read
+            modify_request = {"removeLabelIds": ["UNREAD"]}
+
+            self.service.users().messages().modify(userId="me", id=message_id, body=modify_request).execute()  # type: ignore
+
+            return f"Successfully marked email {message_id} as read. Labels removed: UNREAD"
+
+        except HttpError as error:
+            return f"HTTP Error marking email {message_id} as read: {error}"
+        except Exception as error:
+            return f"Error marking email {message_id} as read: {type(error).__name__}: {error}"
+
+    @authenticate
+    def mark_email_as_unread(self, message_id: str) -> str:
+        """
+        Mark a specific email as unread by adding the 'UNREAD' label.
+        This is useful for flagging emails that need attention or re-processing.
+
+        Args:
+            message_id (str): The ID of the message to mark as unread
+
+        Returns:
+            str: Success message or error description
+        """
+        try:
+            # Add the UNREAD label to mark the email as unread
+            modify_request = {"addLabelIds": ["UNREAD"]}
+
+            self.service.users().messages().modify(userId="me", id=message_id, body=modify_request).execute()  # type: ignore
+
+            return f"Successfully marked email {message_id} as unread. Labels added: UNREAD"
+
+        except HttpError as error:
+            return f"HTTP Error marking email {message_id} as unread: {error}"
+        except Exception as error:
+            return f"Error marking email {message_id} as unread: {type(error).__name__}: {error}"
+
     def _validate_email_params(self, to: str, subject: str, body: str) -> None:
         """Validate email parameters."""
         if not to:
