@@ -7,6 +7,7 @@ from agno.models.message import Message
 
 def get_text_from_message(message: Union[List, Dict, str, Message, BaseModel]) -> str:
     """Return the user texts from the message"""
+    import json
 
     if isinstance(message, str):
         return message
@@ -17,7 +18,15 @@ def get_text_from_message(message: Union[List, Dict, str, Message, BaseModel]) -
         if len(message) == 0:
             return ""
 
-        if "type" in message[0]:
+        # Check if it's a list of Message objects
+        if isinstance(message[0], Message):
+            for m in message:
+                if isinstance(m, Message) and m.role == "user" and m.content is not None:
+                    # Recursively extract text from the message content
+                    content_text = get_text_from_message(m.content)
+                    if content_text:
+                        text_messages.append(content_text)
+        elif "type" in message[0]:
             for m in message:
                 m_type = m.get("type")
                 if m_type is not None and isinstance(m_type, str):
@@ -42,6 +51,8 @@ def get_text_from_message(message: Union[List, Dict, str, Message, BaseModel]) -
     if isinstance(message, dict):
         if "content" in message:
             return get_text_from_message(message["content"])
+        else:
+            return json.dumps(message, indent=2)
     if isinstance(message, Message) and message.content is not None:
         return get_text_from_message(message.content)
     return ""
