@@ -1,2 +1,8 @@
 from agno.vectordb.weaviate.index import Distance, VectorIndex
 from agno.vectordb.weaviate.weaviate import Weaviate
+
+__all__ = [
+    "Distance",
+    "VectorIndex",
+    "Weaviate",
+]
