@@ -226,6 +226,7 @@ class Loop:
         step_input: StepInput,
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
+        stream_events: bool = False,
         stream_intermediate_steps: bool = False,
         stream_executor_events: bool = True,
         workflow_run_response: Optional[WorkflowRunOutput] = None,
@@ -245,7 +246,10 @@ class Loop:
 
         loop_step_id = str(uuid4())
 
-        if stream_intermediate_steps and workflow_run_response:
+        # Considering both stream_events and stream_intermediate_steps (deprecated)
+        stream_events = stream_events or stream_intermediate_steps
+
+        if stream_events and workflow_run_response:
             # Yield loop started event
             yield LoopExecutionStartedEvent(
                 run_id=workflow_run_response.run_id or "",
@@ -266,7 +270,7 @@ class Loop:
         while iteration < self.max_iterations:
             log_debug(f"Loop iteration {iteration + 1}/{self.max_iterations}")
 
-            if stream_intermediate_steps and workflow_run_response:
+            if stream_events and workflow_run_response:
                 # Yield iteration started event
                 yield LoopIterationStartedEvent(
                     run_id=workflow_run_response.run_id or "",
@@ -302,7 +306,7 @@ class Loop:
                     current_step_input,
                     session_id=session_id,
                     user_id=user_id,
-                    stream_intermediate_steps=stream_intermediate_steps,
+                    stream_events=stream_events,
                     stream_executor_events=stream_executor_events,
                     workflow_run_response=workflow_run_response,
                     step_index=composite_step_index,
@@ -363,7 +367,7 @@ class Loop:
                 should_continue = False
                 log_debug(f"Loop ending early due to step termination request at iteration {iteration}")
 
-            if stream_intermediate_steps and workflow_run_response:
+            if stream_events and workflow_run_response:
                 # Yield iteration completed event
                 yield LoopIterationCompletedEvent(
                     run_id=workflow_run_response.run_id or "",
@@ -388,7 +392,7 @@ class Loop:
 
         log_debug(f"Loop End: {self.name} ({iteration} iterations)", center=True, symbol="=")
 
-        if stream_intermediate_steps and workflow_run_response:
+        if stream_events and workflow_run_response:
             # Yield loop completed event
             yield LoopExecutionCompletedEvent(
                 run_id=workflow_run_response.run_id or "",
@@ -523,6 +527,7 @@ class Loop:
         step_input: StepInput,
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
+        stream_events: bool = False,
         stream_intermediate_steps: bool = False,
         stream_executor_events: bool = True,
         workflow_run_response: Optional[WorkflowRunOutput] = None,
@@ -542,7 +547,10 @@ class Loop:
         # Prepare steps first
         self._prepare_steps()
 
-        if stream_intermediate_steps and workflow_run_response:
+        # Considering both stream_events and stream_intermediate_steps (deprecated)
+        stream_events = stream_events or stream_intermediate_steps
+
+        if stream_events and workflow_run_response:
             # Yield loop started event
             yield LoopExecutionStartedEvent(
                 run_id=workflow_run_response.run_id or "",
@@ -563,7 +571,7 @@ class Loop:
         while iteration < self.max_iterations:
             log_debug(f"Async loop iteration {iteration + 1}/{self.max_iterations}")
 
-            if stream_intermediate_steps and workflow_run_response:
+            if stream_events and workflow_run_response:
                 # Yield iteration started event
                 yield LoopIterationStartedEvent(
                     run_id=workflow_run_response.run_id or "",
@@ -599,7 +607,7 @@ class Loop:
                     current_step_input,
                     session_id=session_id,
                     user_id=user_id,
-                    stream_intermediate_steps=stream_intermediate_steps,
+                    stream_events=stream_events,
                     stream_executor_events=stream_executor_events,
                     workflow_run_response=workflow_run_response,
                     step_index=composite_step_index,
@@ -663,7 +671,7 @@ class Loop:
                 should_continue = False
                 log_debug(f"Loop ending early due to step termination request at iteration {iteration}")
 
-            if stream_intermediate_steps and workflow_run_response:
+            if stream_events and workflow_run_response:
                 # Yield iteration completed event
                 yield LoopIterationCompletedEvent(
                     run_id=workflow_run_response.run_id or "",
@@ -688,7 +696,7 @@ class Loop:
 
         log_debug(f"Loop End: {self.name} ({iteration} iterations)", center=True, symbol="=")
 
-        if stream_intermediate_steps and workflow_run_response:
+        if stream_events and workflow_run_response:
             # Yield loop completed event
             yield LoopExecutionCompletedEvent(
                 run_id=workflow_run_response.run_id or "",
