@@ -207,6 +207,7 @@ class Steps:
         session_state: Optional[Dict[str, Any]] = None,
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
+        stream_events: bool = False,
         stream_intermediate_steps: bool = False,
         stream_executor_events: bool = True,
         step_index: Optional[Union[int, tuple]] = None,
@@ -223,7 +224,10 @@ class Steps:
 
         self._prepare_steps()
 
-        if stream_intermediate_steps:
+        # Considering both stream_events and stream_intermediate_steps (deprecated)
+        stream_events = stream_events or stream_intermediate_steps
+
+        if stream_events:
             # Yield steps execution started event
             yield StepsExecutionStartedEvent(
                 run_id=workflow_run_response.run_id or "",
@@ -266,7 +270,7 @@ class Steps:
                     session_id=session_id,
                     user_id=user_id,
                     session_state=session_state,
-                    stream_intermediate_steps=stream_intermediate_steps,
+                    stream_events=stream_events,
                     stream_executor_events=stream_executor_events,
                     workflow_run_response=workflow_run_response,
                     step_index=child_step_index,
@@ -309,7 +313,7 @@ class Steps:
 
             log_debug(f"Steps End: {self.name} ({len(all_results)} results)", center=True, symbol="-")
 
-            if stream_intermediate_steps:
+            if stream_events:
                 # Yield steps execution completed event
                 yield StepsExecutionCompletedEvent(
                     run_id=workflow_run_response.run_id or "",
@@ -438,6 +442,7 @@ class Steps:
         session_state: Optional[Dict[str, Any]] = None,
         session_id: Optional[str] = None,
         user_id: Optional[str] = None,
+        stream_events: bool = False,
         stream_intermediate_steps: bool = False,
         stream_executor_events: bool = True,
         step_index: Optional[Union[int, tuple]] = None,
@@ -454,7 +459,10 @@ class Steps:
 
         self._prepare_steps()
 
-        if stream_intermediate_steps:
+        # Considering both stream_events and stream_intermediate_steps (deprecated)
+        stream_events = stream_events or stream_intermediate_steps
+
+        if stream_events:
             # Yield steps execution started event
             yield StepsExecutionStartedEvent(
                 run_id=workflow_run_response.run_id or "",
@@ -497,7 +505,7 @@ class Steps:
                     session_id=session_id,
                     user_id=user_id,
                     session_state=session_state,
-                    stream_intermediate_steps=stream_intermediate_steps,
+                    stream_events=stream_events,
                     stream_executor_events=stream_executor_events,
                     workflow_run_response=workflow_run_response,
                     step_index=child_step_index,
@@ -540,7 +548,7 @@ class Steps:
 
             log_debug(f"Steps End: {self.name} ({len(all_results)} results)", center=True, symbol="-")
             # Yield steps execution completed event
-            if stream_intermediate_steps:
+            if stream_events:
                 yield StepsExecutionCompletedEvent(
                     run_id=workflow_run_response.run_id or "",
                     workflow_name=workflow_run_response.workflow_name or "",
